﻿using Connected.ServiceModel;

namespace Connected.Interop.Merging;
internal static class PropertyAggregator
{
	public static Dictionary<string, object> Aggregate(params object[] values)
	{
		var result = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);

		for (var i = values.Length - 1; i >= 0; i--)
		{
			if (values[i] is not object value)
				continue;

			foreach (var property in GetImplementedProperties(value))
			{
				if (result.ContainsKey(property.Key))
					continue;

				result.Add(property.Key, property.Value);
			}

			if (value is IPropertyProvider provider)
			{
				foreach (var property in provider.Properties)
				{
					if (result.ContainsKey(property.Key))
						continue;

					result.Add(property.Key, property.Value);
				}
			}
		}

		return result;
	}

	private static Dictionary<string, object> GetImplementedProperties(object value)
	{
		if (value is null)
			return new();

		if (value is object[] objectArray)
		{
			var result = new Dictionary<string, object>();

			for (var i = objectArray.Length - 1; i >= 0; i--)
			{
				var implementations = Properties.GetImplementedProperties(objectArray[i]);

				foreach (var property in implementations)
					result.Add(property.Name, objectArray[i]);
			}

			return result;
		}
		else
		{
			var implementations = Properties.GetImplementedProperties(value);
			var result = new Dictionary<string, object>();

			foreach (var property in implementations)
				result.Add(property.Name, value);

			return result;
		}
	}
}
